---
title: "Tracking In-Cluster"
sub_link: "polyaxon-tracking-api/in-cluster"
meta_title: "Tracking experiments and jobs in-cluster - Polyaxon References"
meta_description: "Tracking experiments and jobs running inside a cluster managed by Polyaxon."
visibility: public
status: published
tags:
    - tracking
    - reference
    - polyaxon
    - client
    - sdk
    - job
sidebar: "polyaxon-tracking-api"
---

## Tracking experiments running inside Polyaxon

For experiment running and managed by Polyaxon, an in-cluster context is exposed to the tracking API 
to transparently detect the experiment context, project, 
experiment group if the experiment belongs to a group, authentication, outputs paths, logs paths, and storage definition.

The only step needed is to create an instance of `Experiment`:

```python
from polyaxon_client.tracking import Experiment

experiment = Experiment()
```

When you start an experiment in-cluster you don't not need to call several of the the exposed methods on the experiments tracking reference, 
because they are recorded automatically, e.g. you don't need to set a name, description, tags, code reference 
because they are saved during the creation of the experiment in the Dashboard or the CLI.

Here an example of how tracking an experiment looks like:

```python

experiment = Experiment()
...
experiment.log_data_ref(data=X, data_name='dataset_X')
experiment.log_data_ref(data=y, data_name='dataset_y')
...
experiment.log_metrics(accuracy_mean=accuracy_mean, accuracy_std=accuracy_std)
...
# If you are using a cloud storage
experiment.log_artifact(file_path)
experiment.log_artifacts(dir_path)
``` 

## Data references

You can always register a hash for every dataset you will use in your experiment.

please refer to the reference [log_params](/references/polyaxon-tracking-api/experiments/#log_data_ref).

## Hyperparams

Since hyperparams are defined in the `params` section of the Polyaxonfile, 
you generally don't need to set them manually. But you can always add or reset these params once your code starts running:

please refer to the reference [log_params](/references/polyaxon-tracking-api/experiments/#log_params).

You can also get the hyperparams that the experiment has:

 * `experiment.get_params`: Returns all the experiment params based on both, 
 this value is also exposed as an env var `POLYAXON_PARAMS`.

    * inputs section
    * params section
    * matrix section

## Metrics

You cab track metric values by providing key-value, the name of the metric and the numeric value. 
You can log as many metrics as you want, Polyaxon will keep track of the history of values for each metric.

```python
experiment.log_metrics(step=1000, loss=0.01, accuracy=0.97)
```  

please refer to the reference [log_metrics](/references/polyaxon-tracking-api/experiments/#log_metrics)

## Artifacts

You can save your outputs and artifacts by:
 * saving them on the outputs path exposed by Polyaxon `get_outputs_path`
 * or if you are using a cloud storage by using [log_artifact](/references/polyaxon-tracking-api/experiments/#log_artifact) 
   and [log_artifacts](/references/polyaxon-tracking-api/experiments/#log_artifacts).

## Get log level: 

If set on the polyaxonfile it will return the log level,
you can access this value by using this function `experiment.get_log_level()`, 
this value is also exposed as an env var `POLYAXON_LOG_LEVEL`.

## Paths

Experiment running inside Polyaxon have some paths and other information exposed to them, to access information about a Polyaxon run:


### Get outputs path: 

The outputs path generated by polyaxon based on the hierarchy of the experiment,
you can access this value by using this function:

```python
experiment.get_outputs_path()
```
 
this value is also exposed as an env var `POLYAXON_RUN_OUTPUTS_PATH`.

        `user/project/group/experiment/files`


### Get outputs refs paths: 

The references outputs paths requested by the user, the order follows the order specified by the user, 
you can access this value by calling this function:

```python
experiment.get_outputs_refs_paths()
```
 
this value is also exposed as an env var `POLYAXON_REFS_OUTPUTS_PATHS`.:

        {
            'jobs': [
                `user/project/job12/files`,
            ], 'experiments': [
                `user/project/group/experiment1/files`,
                `user/project/experiment100/files`
            ]
        }


### Get data paths: 

The data paths mounted for the job/experiment, you can access this value by using this function:

```python
experiment.get_data_paths()
```
 
this value is also exposed as an env var `POLYAXON_RUN_DATA_PATHS`:

        {
          'data1': '/data/1',
          'data-foo': '/data/foo',
        }


## Experiment context

A context of information is exposed as well for experiment running inside Polyaxon with some information if the experiment is distributed:

 * `experiment.get_cluster_def`: Returns cluster definition created by polyaxon, 
 this value is also exposed as an env var `POLYAXON_CLUSTER`.
    ```json
    {
        "master": ["plx-master0-8eefb7a1146f476ca66e3bee9b88c1de:2000"],
        "worker": ["plx-worker1-8eefb7a1146f476ca66e3bee9b88c1de:2000",
                   "plx-worker2-8eefb7a1146f476ca66e3bee9b88c1de:2000"],
        "ps": ["plx-ps3-8eefb7a1146f476ca66e3bee9b88c1de:2000"],
    }
    ```

 * `experiment.get_tf_config`: Returns the TF_CONFIG defining the cluster and the current task, 
    if the experiment is running a distributed tensorflow graph.
    if `envvar` is not null, it will set and env variable with `envvar`.

 * `experiment.get_experiment_info`: Returns information about the experiment, 
 this value is also exposed as an env var `POLYAXON_EXPERIMENT_INFO`.

    * project_name
    * experiment_group_name
    * experiment_name
    * project_uuid
    * experiment_group_uuid
    * experiment_uuid

 * `experiment.get_task_info`: Returns the task info: `{"type": str, "index": int}`, 
 this value is also exposed as an env var `POLYAXON_TASK_INFO`.


## Accessing the API

Since the experiment is running in-cluster, the tracking API knows how to instantiate a client, 
this client is used not only to log the previous information, 
but it could be used to access to much richer API.

```python
client = experiment.client
```

This client is authenticated with the current user, and gives scoped access to all accessible objects of the user.

Please look at [Polyaxon client](/references/polyaxon-client/) for more information.
